@@ -73,17 +73,23 @@ def _doNothing(canvas, doc):
     pass
 
 class PTCycle(list):
-    def __init__(self):
+    def __new__(cls,*args,**kwds):
+        self = list.__new__(cls,*args,**kwds)
         self._restart = 0
         self._idx = 0
-        list.__init__(self)
+        return self
 
-    def cyclicIterator(self):
-        while 1:
-            yield self[self._idx]
-            self._idx += 1
-            if self._idx>=len(self):
-                self._idx = self._restart
+    @property
+    def next_value(self):
+        v = self[self._idx]
+        self._idx += 1
+        if self._idx>=len(self):
+            self._idx = self._restart
+        return v
+
+    @property
+    def peek(self):
+        return self[self._idx]
 
 class IndexingFlowable(Flowable):
     """Abstract interface definition for flowables which might
@@ -225,12 +231,12 @@ class Indenter(FrameActionFlowable):
         frame._rightExtraIndent += self.right
 
 class NotAtTopPageBreak(FrameActionFlowable):
-    def __init__(self):
-        pass
+    def __init__(self,nextTemplate=None):
+        self.nextTemplate = nextTemplate
 
     def frameAction(self,frame):
         if not frame._atTop:
-            frame.add_generated_content(PageBreak())
+            frame.add_generated_content(PageBreak(nextTemplate=self.nextTemplate))
 
 class NextPageTemplate(ActionFlowable):
     """When you get to the next page, use the template specified (change to two column, for example)  """
@@ -291,7 +297,7 @@ class PageTemplate:
 
 def _addGeneratedContent(flowables,frame):
     S = getattr(frame,'_generated_content',None)
-    if S: 
+    if S:
         flowables[0:0] = S
         del frame._generated_content
 
@@ -372,7 +378,7 @@ class BaseDocTemplate:
     First attempt at defining a document template class.
 
     The basic idea is simple.
-    
+
     1)  The document has a list of data associated with it
         this data should derive from flowables. We'll have
         special classes like PageBreak, FrameBreak to do things
@@ -417,7 +423,7 @@ class BaseDocTemplate:
     - bottomMargin:  Margin sizes in points (default 1 inch).  These margins may be
       overridden by the pageTemplates.  They are primarily of interest for the
       SimpleDocumentTemplate subclass.
-    
+
     - allowSplitting:  If set flowables (eg, paragraphs) may be split across frames or pages
       (default: 1)
     - title: Internal title for document (does not automatically display on any page)
@@ -545,6 +551,27 @@ class BaseDocTemplate:
         self.frame._debug = self._debug
         self.handle_frameBegin()
 
+    def _setPageTemplate(self):
+        if hasattr(self,'_nextPageTemplateCycle'):
+            #they are cycling through pages'; we keep the index
+            self.pageTemplate = self._nextPageTemplateCycle.next_value
+        elif hasattr(self,'_nextPageTemplateIndex'):
+            self.pageTemplate = self.pageTemplates[self._nextPageTemplateIndex]
+            del self._nextPageTemplateIndex
+        elif self.pageTemplate.autoNextPageTemplate:
+            self.handle_nextPageTemplate(self.pageTemplate.autoNextPageTemplate)
+            self.pageTemplate = self.pageTemplates[self._nextPageTemplateIndex]
+
+    def _samePT(self,npt):
+        if isSeq(npt):
+            return getattr(self,'_nextPageTemplateCycle',[])
+        if isinstance(npt,strTypes):
+            return npt == (self.pageTemplates[self._nextPageTemplateIndex].id if hasattr(self,'_nextPageTemplateIndex') else self.pageTemplate.id)
+        if isinstance(npt,int) and 0<=npt<len(self.pageTemplates):
+            if hasattr(self,'_nextPageTemplateIndex'):
+                return npt==self._nextPageTemplateIndex
+            return npt==self.pageTemplates.find(self.pageTemplate)
+
     def handle_pageEnd(self):
         ''' show the current page
             check the next page template
@@ -575,16 +602,7 @@ class BaseDocTemplate:
             if self._debug: logger.debug("ending page %d" % self.page)
             self.canv.setPageRotation(getattr(self.pageTemplate,'rotation',self.rotation))
             self.canv.showPage()
-
-            if hasattr(self,'_nextPageTemplateCycle'):
-                #they are cycling through pages'; we keep the index
-                self.pageTemplate = next(self._nextPageTemplateCycle)
-            elif hasattr(self,'_nextPageTemplateIndex'):
-                self.pageTemplate = self.pageTemplates[self._nextPageTemplateIndex]
-                del self._nextPageTemplateIndex
-            elif self.pageTemplate.autoNextPageTemplate:
-                self.handle_nextPageTemplate(self.pageTemplate.autoNextPageTemplate)
-                self.pageTemplate = self.pageTemplates[self._nextPageTemplateIndex]
+            self._setPageTemplate()
             if self._emptyPages==0:
                 pass    #store good state here
         self._hanging.append(PageBegin)
@@ -664,7 +682,7 @@ class BaseDocTemplate:
                 raise ValueError("Invalid cycle restart position")
 
             #ensure we start on the first one
-            self._nextPageTemplateCycle = c.cyclicIterator()
+            self._nextPageTemplateCycle = c
         else:
             raise TypeError("argument pt should be string or integer or list")
 
@@ -758,6 +776,11 @@ class BaseDocTemplate:
             return
 
         if isinstance(f,PageBreak):
+            npt = f.nextTemplate
+            if npt and not self._samePT(npt):
+                npt=NextPageTemplate(npt)
+                npt.apply(self)
+                self.afterFlowable(npt)
             if isinstance(f,SlowPageBreak):
                 self.handle_pageBreak(slow=1)
             else:
@@ -832,7 +855,7 @@ class BaseDocTemplate:
                                 pageCompression=self.pageCompression,
                                 enforceColorSpace=self.enforceColorSpace,
                                 )
- 
+
         getattr(self.canv,'setEncrypt',lambda x: None)(self.encrypt)
 
         self.canv._cropMarks = self.cropMarks
@@ -884,6 +907,13 @@ class BaseDocTemplate:
         try:
             canv._doctemplate = self
             while len(flowables):
+                if self._hanging and self._hanging[-1] is PageBegin and isinstance(flowables[0],PageBreakIfNotEmpty):
+                    npt = flowables[0].nextTemplate
+                    if npt and not self._samePT(npt):
+                        npt=NextPageTemplate(npt)
+                        npt.apply(self)
+                        self._setPageTemplate()
+                    del flowables[0]
                 self.clean_hanging()
                 try:
                     first = flowables[0]
@@ -929,7 +959,14 @@ class BaseDocTemplate:
     def notify(self, kind, stuff):
         """Forward to any listeners"""
         for l in self._indexingFlowables:
-            l.notify(kind, stuff)
+            _canv = getattr(l,'_canv',self)
+            try:
+                if _canv==self:
+                    l._canv = self.canv
+                l.notify(kind, stuff)
+            finally:
+                if _canv==self:
+                    del l._canv
 
     def pageRef(self, label):
         """hook to register a page number"""
@@ -943,7 +980,7 @@ class BaseDocTemplate:
                    ):
         """Makes multiple passes until all indexing flowables
         are happy.
-        
+
         Returns number of passes"""
         self._indexingFlowables = []
         #scan the story and keep a copy
@@ -990,7 +1027,7 @@ class BaseDocTemplate:
         del self._multiBuildEdits
         if verbose: print('saved')
         return passes
-        
+
     #these are pure virtuals override in derived classes
     #NB these get called at suitable places by the base class
     #so if you derive and override the handle_xxx methods
